#coding: utf-8

# -------------------------------------------------------------------
# Énumération de bijections
#
# Marc Lorenzi
# 17 novembre 2021
# -------------------------------------------------------------------

# Documentation : Voir le notebook sur l'énumération des permutations.

def pivot(s):
    n = len(s)
    i = n - 1
    while i > 0 and s[i] < s[i - 1]: i = i - 1
    return i

def next_piv(s):
    i = pivot(s)
    if i > 0:
        j = len(s) - 1
        while s[j] < s[i - 1]: 
            j = j - 1
        return (i, j)
    else: return (0, 0)

def renverser(s, i):
    n = len(s)
    j = i
    k = n - 1
    while j < k:
        s[j], s[k] = s[k], s[j]
        j = j + 1
        k = k - 1

def successeur(s):
    s1 = s[:]
    i, j = next_piv(s1)
    if i > 0: 
        s1[j], s1[i - 1] = s1[i - 1], s1[j]
        renverser(s1, i)
        return s1
    else: return None

# Énumérer les permutations de [0, ..., n - 1]

def permutations(n):
    s = list(range(n))
    while True:
        yield s
        s = successeur(s)
        if s == None: break

# Énumérer les bijections de E sur F

def bijections(E, F):
    m, n = len(E), len(F)
    if m != n: return
    for s in permutations(n):
        f = dict()
        for k in range(n):
            f[E[k]] = F[s[k]]
        yield f